<?php

/*
Plugin Name: 清理冗余
Plugin URI: https://zmingcx.com
Author:  Robin
Description: 清理不需要的冗余数据，保持数据库干净快速。
Version: 1.0
Author URI: https://zmingcx.com
*/

if ( ! defined( 'ABSPATH' ) ){
	exit;
}

function zm_wp_cleaner_settings_link( $action_links, $plugin_file ) {
	if( $plugin_file==plugin_basename( __FILE__) ) {
		$wcu_settings_link = '<a href="options-general.php?page=' . dirname( plugin_basename( __FILE__ ) ) . '/zm-wp-cleaner.php">' . __( "清理" ) . '</a>';
		array_unshift( $action_links, $wcu_settings_link );
	}
	return $action_links;
}
add_filter( 'plugin_action_links','zm_wp_cleaner_settings_link', 10, 2 );

function zm_wp_cleaner_admin() {
	add_options_page( '清理冗余', '<span class="dashicons dashicons-trash" style="font-size: 15px;vertical-align: -12%;"></span>清理冗余','manage_options', __FILE__, 'zm_wp_cleaner_page' );
}
function zm_wp_cleaner_page() {
?>

<?php
function zm_wp_cleaner( $type ) {
	global $wpdb;
	switch( $type ) {
		case "revision":
			$zwc_sql = "DELETE FROM $wpdb->posts WHERE post_type = 'revision'";
			$wpdb->query($zwc_sql);
			break;
		case "draft":
			$zwc_sql = "DELETE FROM $wpdb->posts WHERE post_status = 'draft'";
			$wpdb->query($zwc_sql);
			break;
		case "autodraft":
			$zwc_sql = "DELETE FROM $wpdb->posts WHERE post_status = 'auto-draft'";
			$wpdb->query($zwc_sql);
			break;
		case "moderated":
			$zwc_sql = "DELETE FROM $wpdb->comments WHERE comment_approved = '0'";
			$wpdb->query($zwc_sql);
			break;
		case "spam":
			$zwc_sql = "DELETE FROM $wpdb->comments WHERE comment_approved = 'spam'";
			$wpdb->query($zwc_sql);
			break;
		case "trash":
			$zwc_sql = "DELETE FROM $wpdb->comments WHERE comment_approved = 'trash'";
			$wpdb->query($zwc_sql);
			break;
		case "postmeta":
			$zwc_sql = "DELETE pm FROM $wpdb->postmeta pm LEFT JOIN $wpdb->posts wp ON wp.ID = pm.post_id WHERE wp.ID IS NULL";
			$wpdb->query($zwc_sql);
			break;
		case "commentmeta":
			$zwc_sql = "DELETE FROM $wpdb->commentmeta WHERE comment_id NOT IN (SELECT comment_id FROM $wpdb->comments)";
			$wpdb->query($zwc_sql);
			break;
		case "relationships":
			$zwc_sql = "DELETE FROM $wpdb->term_relationships WHERE term_taxonomy_id=1 AND object_id NOT IN (SELECT id FROM $wpdb->posts)";
			$wpdb->query($zwc_sql);
			break;
		case "feed":
			$zwc_sql = "DELETE FROM $wpdb->options WHERE option_name LIKE '_site_transient_browser_%' OR option_name LIKE '_site_transient_timeout_browser_%' OR option_name LIKE '_transient_feed_%' OR option_name LIKE '_transient_timeout_feed_%'";
			$wpdb->query($zwc_sql);
			break;
	}
}

function zm_wp_cleaner_count($type){
	global $wpdb;
	switch($type){
		case "revision":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->posts WHERE post_type = 'revision'";
			$count = $wpdb->get_var($zwc_sql);
			break;
		case "draft":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->posts WHERE post_status = 'draft'";
			$count = $wpdb->get_var($zwc_sql);
			break;
		case "autodraft":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->posts WHERE post_status = 'auto-draft'";
			$count = $wpdb->get_var($zwc_sql);
			break;
		case "moderated":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->comments WHERE comment_approved = '0'";
			$count = $wpdb->get_var($zwc_sql);
			break;
		case "spam":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->comments WHERE comment_approved = 'spam'";
			$count = $wpdb->get_var($zwc_sql);
			break;
		case "trash":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->comments WHERE comment_approved = 'trash'";
			$count = $wpdb->get_var($zwc_sql);
			break;
		case "postmeta":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->postmeta pm LEFT JOIN $wpdb->posts wp ON wp.ID = pm.post_id WHERE wp.ID IS NULL";
			$count = $wpdb->get_var($zwc_sql);
			break;
		case "commentmeta":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->commentmeta WHERE comment_id NOT IN (SELECT comment_id FROM $wpdb->comments)";
			$count = $wpdb->get_var($zwc_sql);
			break;
		case "relationships":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->term_relationships WHERE term_taxonomy_id=1 AND object_id NOT IN (SELECT id FROM $wpdb->posts)";
			$count = $wpdb->get_var($zwc_sql);
			break;
		case "feed":
			$zwc_sql = "SELECT COUNT(*) FROM $wpdb->options WHERE option_name LIKE '_site_transient_browser_%' OR option_name LIKE '_site_transient_timeout_browser_%' OR option_name LIKE '_transient_feed_%' OR option_name LIKE '_transient_timeout_feed_%'";
			$count = $wpdb->get_var($zwc_sql);
			break;
	}
	return $count;
}

function zm_wp_cleaner_optimize(){
	global $wpdb;
	$zwc_sql = 'SHOW TABLE STATUS FROM `'.DB_NAME.'`';
	$result = $wpdb->get_results($zwc_sql);
	foreach($result as $row){
		$zwc_sql = 'OPTIMIZE TABLE '.$row->Name;
		$wpdb->query($zwc_sql);
	}
}

	$zwc_message = '';

	if(isset($_POST['zm_wp_cleaner_revision'])){
		zm_wp_cleaner('revision');
		$zwc_message = "所有修订已删除";
	}

	if(isset($_POST['zm_wp_cleaner_draft'])){
		zm_wp_cleaner('draft');
		$zwc_message = "所有草稿已删除";
	}

	if(isset($_POST['zm_wp_cleaner_autodraft'])){
		zm_wp_cleaner('autodraft');
		$zwc_message = "所有自动草稿已删除";
	}
	
	if(isset($_POST['zm_wp_cleaner_moderated'])){
		zm_wp_cleaner('moderated');
		$zwc_message = "所有审核的评论已删除";
	}

	if(isset($_POST['zm_wp_cleaner_spam'])){
		zm_wp_cleaner('spam');
		$zwc_message = "所有垃圾评论已删除";
	}

	if(isset($_POST['zm_wp_cleaner_trash'])){
		zm_wp_cleaner('trash');
		$zwc_message = "所有垃圾评论已删除";
	}

	if(isset($_POST['zm_wp_cleaner_postmeta'])){
		zm_wp_cleaner('postmeta');
		$zwc_message = "所有无关联的文章信息已删除";
	}

	if(isset($_POST['zm_wp_cleaner_commentmeta'])){
		zm_wp_cleaner('commentmeta');
		$zwc_message = "所有无关联的评论信息已删除";
	}

	if(isset($_POST['zm_wp_cleaner_relationships'])){
		zm_wp_cleaner('relationships');
		$zwc_message = "所有无关联的信息已删除";
	}

	if(isset($_POST['zm_wp_cleaner_feed'])){
		zm_wp_cleaner('feed');
		$zwc_message = "所有仪表盘消息已删除";
	}

	if(isset($_POST['zm_wp_cleaner_all'])){
		zm_wp_cleaner('revision');
		zm_wp_cleaner('draft');
		zm_wp_cleaner('autodraft');
		zm_wp_cleaner('moderated');
		zm_wp_cleaner('spam');
		zm_wp_cleaner('trash');
		zm_wp_cleaner('postmeta');
		zm_wp_cleaner('commentmeta');
		zm_wp_cleaner('relationships');
		zm_wp_cleaner('feed');
		$zwc_message = "所有不需要的数据已删除";
	}

	if($zwc_message != ''){
		echo '<div id="message" class="updated"><p><strong>' . $zwc_message . '</strong></p></div>';
	}

?>

	<div class="wrap">
		<h2><span class="dashicons dashicons-trash" style="font-size: 28px;color: #555;margin-right: 10px;vertical-align: -22%;"></span>清理冗余</h2>
		<p>用于清理冗余的数据，操作不可逆，请提前作好备份！</p>

		<table class="widefat">
			<thead>
				<tr>
					<th>类型</th>
					<th>计数</th>
					<th>操作</th>
				</tr>
			</thead>
			<tbody id="the-list">
				<tr class="alternate">
					<td class="column-name">修订</td>
					<td class="column-name"><?php echo zm_wp_cleaner_count('revision'); ?></td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_revision" value="revision" />
							<?php if( zm_wp_cleaner_count( 'revision' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除" />
							<?php } ?>
						</form>
					</td>
				</tr>
				<tr>
					<td class="column-name">草稿</td>
					<td class="column-name"><?php echo zm_wp_cleaner_count('draft'); ?></td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_draft" value="draft" />
							<?php if( zm_wp_cleaner_count( 'draft' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除（酌情删除）" />
							<?php } ?>
						</form>
					</td>
				</tr>
				<tr class="alternate">
					<td class="column-name">自动草稿</td>
					<td class="column-name"><?php echo zm_wp_cleaner_count('autodraft'); ?></td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_autodraft" value="autodraft" />
							<?php if( zm_wp_cleaner_count( 'autodraft' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除" />
							<?php } ?>
						</form>
					</td>
				</tr>
				<tr>
					<td class="column-name">审核的评论</td>
					<td class="column-name"><?php echo zm_wp_cleaner_count('moderated'); ?></td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_moderated" value="moderated" />
							<?php if( zm_wp_cleaner_count( 'moderated' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除" />
							<?php } ?>
						</form>
					</td>
				</tr>
				<tr class="alternate">
					<td class="column-name">垃圾评论</td>
					<td class="column-name"><?php echo zm_wp_cleaner_count('spam'); ?></td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_spam" value="spam" />
							<?php if( zm_wp_cleaner_count( 'spam' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除" />
							<?php } ?>
						</form>
					</td>
				</tr>
				<tr>
					<td class="column-name">垃圾邮件评论</td>
					<td class="column-name"><?php echo zm_wp_cleaner_count('trash'); ?></td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_trash" value="trash" />
							<?php if( zm_wp_cleaner_count( 'trash' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除" />
							<?php } ?>
						</form>
					</td>
				</tr>
				<tr class="alternate">
					<td class="column-name">无关联的文章信息</td>
					<td class="column-name">
						<?php echo zm_wp_cleaner_count('postmeta'); ?>
					</td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_postmeta" value="postmeta" />
							<?php if( zm_wp_cleaner_count( 'postmeta' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除" />
							<?php } ?>
						</form>
					</td>
				</tr>
				<tr>
					<td class="column-name">无关联的评论信息</td>
					<td class="column-name">
						<?php echo zm_wp_cleaner_count('commentmeta'); ?>
					</td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_commentmeta" value="commentmeta" />
							<?php if( zm_wp_cleaner_count( 'commentmeta' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除" />
							<?php } ?>
						</form>
					</td>
				</tr>
				<tr class="alternate">
					<td class="column-name">无关联的信息</td>
					<td class="column-name">
						<?php echo zm_wp_cleaner_count('relationships'); ?>
					</td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_relationships" value="relationships" />
							<?php if( zm_wp_cleaner_count( 'relationships' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除" />
							<?php } ?>
						</form>
					</td>
				</tr>
				<tr>
					<td class="column-name">仪表盘消息</td>
					<td class="column-name"><?php echo zm_wp_cleaner_count('feed'); ?></td>
					<td class="column-name">
						<form action="" method="post">
							<input type="hidden" name="zm_wp_cleaner_feed" value="feed" />
							<?php if( zm_wp_cleaner_count( 'feed' ) > 0 ) { ?>
								<input type="submit" class="button-primary" value="删除" />
							<?php } ?>
						</form>
					</td>
				</tr>
			</tbody>
		</table>
		</p>

		<!-- 
		<p>
		<form action="" method="post">
			<input type="hidden" name="zm_cleaner_all" value="all" />
			<input type="submit" class="button-primary" value="全部删除" />
		</form>
		</p>
		-->
	</div>
<?php
}
add_action('admin_menu', 'zm_wp_cleaner_admin');